<?php


namespace TDevAgency\DeliveriesPayments\Traits;


use App\Models\Post;
use Exception;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Str;
use Mcamara\LaravelLocalization\Facades\LaravelLocalization;
use RuntimeException;
use TDevAgency\Erp\Models\CoreConfig;

trait DeliveryPayment
{

    private static $_instances = [];
    private $code;
    private $config;
    private $coreConfig;

    private function __construct($code, $type)
    {
        $conteiner_id = null;

        if (Config::has('container_id')) {
            $conteiner_id = Config::get('container_id');
        } elseif (!empty(Auth::user()->container)) {
            $conteiner_id = Auth::user()->container->id;
        }

        $this->coreConfig = Cache::rememberForever(
            'core_config_' . $conteiner_id,
            function () {
                return CoreConfig::container()->get()->mapWithKeys(
                    function (CoreConfig $coreConfig) {
                        return [
                            $coreConfig->config_key => $coreConfig->toArray()
                        ];
                    }
                )->toArray();
            }
        );
        $this->code       = $code;
        $this->config     = Config::get('deliveries_payments.' . $type . '.' . $code);
    }

    /**
     * @param $type
     * @param $name
     * @return $this
     */
    public static function instance($type, $name): self
    {
        if (!isset(self::$_instances[$type][$name])) {
            self::$_instances[$type][$name] = new static($name, $type);
        }
        return self::$_instances[$type][$name];
    }

    /**
     * Одиночки не должны быть клонируемыми.
     */
    protected function __clone()
    {
    }

    /**
     * Одиночки не должны быть восстанавливаемыми из строк.
     * @throws Exception
     */
    public function __wakeup()
    {
        throw new RuntimeException("Cannot unserialize a singleton.");
    }

    /**
     * @return array
     */
    final public function getAdminOptions(): array
    {
        $options = [
            ['view'        => 'select2',
             'key'         => $this->getCoreConfigKey('is_active'),
             'validate'    => 'required',
             'placeholder' => trans('cms::admin.status'),
             'options'     => [
                 0 => trans('cms::admin.status_inactive'),
                 1 => trans('cms::admin.status_active'),
             ],
             'value'       => $this->getCoreConfigValue('is_active') ?? 0
            ],

        ];

        foreach (LaravelLocalization::getSupportedLocales() as $code => $locale) {
            $options[] = [
                'view'        => 'input',
                'key'         => $this->getCoreConfigKey('title_' . $code),
                'value'       => $this->getCoreConfigValue('title_' . $code),
                'placeholder' => 'Название на ' . $locale['native']
            ];
        }


        return array_merge($options, $this->getConfig('admin_options') ?? [], $this->prepareAdditionalAdminOptions());
    }

    /**
     * Get payment driver key
     *
     * @return string
     */
    public function getCode(): string
    {
        return $this->code;
    }

    /**
     * @param null $name
     * @return mixed|null
     */
    final public function getConfig($name = null)
    {
        if ($name === null) {
            return $this->config;
        }

        return $this->config[$name] ?? null;
    }

    /**
     * @param $key
     * @return string
     */
    final public function getCoreConfigKey($key): string
    {
        return sprintf('%s_%s_%s', $this->config['model'], $this->code, Str::slug($key, '_'));
    }

    /**
     * @param $key
     * @return mixed
     */
    final public function getCoreConfigValue($key)
    {
        return $this->coreConfig[$this->getCoreConfigKey($key)]['config_value'] ?? null;
    }


    /**
     * @return mixed|null
     */
    public function getDisplayName()
    {
        return $this->getCoreConfigValue('title_' . LaravelLocalization::getCurrentLocale());
    }

    /**
     * Return system payment name
     *
     * @return string
     */
    public function getLabel(): string
    {
        if (isset($this->config['label'][App::getLocale()])) {
            return $this->config['label'][App::getLocale()];
        }

        if (isset($this->config['label']['en'])) {
            return $this->config['label']['en'];
        }

        return reset($this->config['label']);
    }

    final public function getSuffix($key): ?string
    {
        return str_replace(sprintf('%s_%s_', $this->config['model'], $this->code), '', $key);
    }

    /**
     * Check payment is active for store of container
     *
     * @return bool
     */
    public function isActive(): bool
    {
        return $this->getCoreConfigValue('is_active') ?? 0;
    }

}
